; punkoe
; punkoe.asm - initialisations

  section ".text"

  include "os.h"
  xdef _nostub
  xdef _ti89
  xdef _ti92plus
_v200:
  xdef _v200

  include "punkoe.h"

; Dbut du programme :
debut:

; Commentaire (standard _nostub) :
  move.l  (a7),(a7)
  dc.w    $6000 ; bra.w
lbl_bra:
  dc.w    prog-lbl_bra
  dc.w    $2e76,$5c7b,$4e74,$4e72,$4afc,0 ; Signature

  dc.l    $01000000 ; standard_revision
  dc.w    3 ; num_extensions

  dc.w    $0 ; comment
  dc.w    comment_str-debut

  dc.w    $1 ; program name
  dc.w    name_str-debut

  dc.w    $3 ; program version
  dc.w    version-debut

comment_str:
  dc.b    "Arcade game",0
name_str:
  dc.b    "Punkoe",0
  even
version:
  dc.b    0,9,8,0

; Initialisations, allocation d'espace pour l'cran virtuel, la map
; et les differentes variables utilisees dans le jeu (au total, ~7 ko).
; Substitution des interruptions

prog:
  movem.l d3-d7/a2-a6,-(a7)

; Detection du modele de TI (89,92,V200)
; En fonction de cela, patcher le code pour que l'affichage soit
; centre et que la touche de saut soit [HAND] sur les TI a grand ecran 
; et remettre l'affichage au coin en haut a gauche et la touche de saut
; [2nd] pour les TI a petit ecran
  movea.l ($C8).w,a0
  movea.l ScrRect*4(a0),a0
  cmpi.b  #159,2(a0)
  bne.s   \PatchTI92_V200

  ; Le modele a un petit ecran
    move.w  #$4C00,d0
    moveq.l #4,d1
    bra.s   \DoPatch

  \PatchTI92_V200:
  ; Le modele a un grand ecran
    move.w  #$4C00+(30*(128-96)+(240-160-16)/8)/2,d0
    moveq.l #3,d1

\DoPatch:
  lea.l   PatchEcran+2(pc),a0
  move.w  d0,(a0)
  lea.l   PatchLectureTouches+2(pc),a0
  move.w  d1,(a0)
  move.w  d1,12(a0)
;  move.w  d1,Patch2LectureTouches-PatchLectureTouches(a0)

; Sauvegarde de l'ecran sur la pile
  move.w  #3840/4-1,d0
  movea.w #$4C00,a0
  \SaveScr:
    move.l  (a0),-(a7) ; sauve l'ecran
    clr.l   (a0)+ ; puis l'efface
    dbf     d0,\SaveScr

; Substitution des interruptions
  move.l  (AI1).w,-(a7) ; Sauve l'adresse du handler de l'AI1
  move.l  (AI5).w,-(a7) ; idem pour l'AI5

  movea.l #$600001,a1
  bclr.b  #2,(a1) ; dsactivation de la protection de la mmoire basse
  lea.l   counter_handler(pc),a0 ; Mise en place de mon propre
  move.l  a0,(AI1).w ;             handler pour l'AI1
  lea.l   rte_handler(pc),a0 ; Redirection de l'AI5
  move.l  a0,(AI5).w ;         vers un rte.
  bset.b  #2,(a1) ; ractivation de la protection de la mmoire basse

; Allocation de l'ecran virtuel
  move.w  #(96*30)/4-1,d0
  \ClrVEcran:
    clr.l   -(a7)
    dbf     d0,\ClrVEcran
  movea.l a7,a6 ; a6 = addr du buffer de 2880 octets alloue sur la pile

; Allocation de la map
  movea.l a7,a0
  lea.l   -10*5(a7),a7
  movea.l a7,a5 ; a5 = addr de la map

; Initialisation de la map
  moveq.l #(10*5)/2-1,d0
  \EffaceMap:
    clr.w   -(a0)
    dbf     d0,\EffaceMap

  moveq.l #10*2-1,d0
  \AjoutePlateForme:
    addq.b  #1,(a0)+
    dbf     d0,\AjoutePlateForme

; Allocation et initialisation des variables
  moveq.l #10,d7 ; hauteur de la camera (hcam)
  moveq.l #24*4,d6 ; hauteur du perso (hperso)
  moveq.l #24*5,d5 ; hauteur de la map (hmap)
  moveq.l #0,d4 ; valeur de la barre de saut (valbarre)

  lea.l   -taille_variables(a7),a7
  movea.l a7,a4 ; a4 = addr des variables
  movea.l a4,a0
  clr.w   (a0)+ ; xperso
  move.w  #1,(a0)+ ; dirperso
  clr.l   (a0)+ ; etatperso = COURT  et  etatbarre = VIDE
  clr.w   (a0)+ ; even_saut
  move.l  #$000A0003,(a0)+ ; hcam_mod24 et nbmaxtrous
;  move.w  #10,(a0)+ ; hcam_mod24
;  move.w  #3,(a0)+ ; nbmaxtrous
; t_vperso et randseed ne sont pas initialisees

; Initialisation des timers
  lea.l   timers(pc),a3
  clr.l   (a3)
  clr.l   4(a3)

; A ce point du programme, toutes les initialisations et allocations
; ont eu lieu. Recapitulation :
; a6 = ecran virtuel (3000 octets)
; a5 = map (10*5=50 octets)
; a4 = variables (taille_variables octets)
; a3 = timers
; d7 = hcam
; d6 = hperso
; d5 = hmap
; d4 = valbarre

; Toutes les fonctions appelees dans la boucle principale peuvent
; dtruire d0-d3/a0-a2

  bsr     Afficher

\MainLoop:

  bsr     GestionBarreSaut

  bsr     GestionPerso

  bsr     GestionCamera

  bsr     Afficher

  cmp.w   d7,d6 ; hperso - hcam
  bgt.s   \MainLoop

; Liberation de l'ecran virtuel, de la map et des variables
  lea.l  2880+10*5+taille_variables(a7),a7

; Restauration des interruptions
  movea.l #$600001,a1
  bclr.b  #2,(a1) ; dsactivation de la protection de la mmoire basse
  move.l  (a7)+,(AI5).w ; Restaure l'AI5
  move.l  (a7)+,(AI1).w ; Restaure l'AI1
  bset.b  #2,(a1) ; ractivation de la protection de la mmoire basse

  lea.l   -40(a7),a7 ; buffer pour stocker la phrase de fin

; Comparaison du score actuel et du meilleur score
  lea.l   pas_high_score_str(pc),a1
  lea.l   high_score(pc),a0
  move.w  (a0),-(a7) ; empile le meilleur score et
  move.w  d7,-(a7) ; le score actuel pour les afficher
  cmp.w   (a0),d7
  bls.s   \PasDeNouveauHighScore
    move.w  d7,(a0) ; mise  jour du high score
    lea.l   new_high_score_str(pc),a1
\PasDeNouveauHighScore:

  move.l  a1,-(a7) ; phrase
  pea.l   8(a7) ; buffer

  movea.l ($C8).w,a2
  movea.l sprintf*4(a2),a0
  jsr     (a0)

  pea.l   12(a7)
  movea.l ST_helpMsg*4(a2),a0
  jsr     (a0)

  movea.l ngetchx*4(a2),a0
  jsr     (a0)

  lea.l   16+40(a7),a7

; Restauration de l'cran
  move.w  #3840/4-1,d0
  movea.w #$4C00+3840,a0
  \RestScr:
    move.l  (a7)+,-(a0)
    dbf     d0,\RestScr

  movem.l (a7)+,d3-d7/a2-a6

  rts

counter_handler:
  move.l  a0,-(a7)
  lea.l   timers(pc),a0
  addq.w  #1,(a0)+ ; vperso
  addq.w  #1,(a0)+ ; hcam
  addq.w  #1,(a0)+ ; xperso
  addq.w  #1,(a0)+ ; barre
  movea.l (a7)+,a0
rte_handler:
  rte
